/*
 * Simply RISC S1 Definitions
 *
 * (C) Copyleft 2008 Simply RISC LLP
 * AUTHORS: Cammisuli Salvatore <salcam@opencores.org>
 *          Di Paola Nunzio <ingnuccio@opencores.org>
 *
 * LICENSE:
 * This is a Free Hardware Design; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * The above named program is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * DESCRIPTION:
 * Simple constant definitions used by the S1 Core design.
 */

`include "t1_defs.h"
`timescale 1ns/100ps

`define FPGA_SYN
`define FPGA_SYN_1THREAD
`define FPGA_SYN_NO_SPU

// Size of the buses
`define AHB_ADDR_WIDTH  64
`define AHB_DATA_WIDTH  64
`define AHB_SIZE_WIDTH   3
`define AHB_BURST_WIDTH  3
`define AHB_PROT_WIDTH   4
`define AHB_TRANS_WIDTH  2

// States of the FSM of the bridge
`define STATE_WAKEUP          4'b00
`define STATE_ADDRESS_DATA    4'b01
`define STATE_WAITED          4'b10

// Transfer types
`define AHB_IDLE   2'b00
`define AHB_BUSY   2'b01
`define AHB_NONSEQ 2'b10
`define AHB_SEQ    2'b11

// Burst signal encoding
`define AHB_SINGLE     3'b000
`define AHB_INCR       3'b001
`define AHB_WRAP4      3'b010
`define AHB_INCR4      3'b011
`define AHB_WRAP8      3'b100
`define AHB_INCR8      3'b101
`define AHB_WRAP16     3'b110
`define AHB_INCR16     3'b111

// HRESP Signal
`define AHB_OKAY     1'b0
`define AHB_ERROR    1'b1

// Constants used by the timer of the Reset Controller
`define TIMER_BITS 16
`define RESET_CYCLES_1   100
`define RESET_CYCLES_2  1000
`define RESET_CYCLES_3  2000
`define RESET_CYCLES_4  3000
`define GCLK_CYCLES      900

